import matplotlib.pyplot as plt
import mne
import numpy as np
from mnelab.io import read_raw
from rich import print
from rich.table import Table
from pyxdf import match_streaminfos, resolve_streams


def summary(x):
    """Return summary statistics of one-dimensional array x."""
    return (np.min(x), np.percentile(x, 25), np.mean(x), np.median(x),
            np.percentile(x, 75), np.max(x))

file_name = "EPrime_11.xdf"
# file_name = "Presentation_26.xdf"
# file_name = "Presentation_27.xdf"
# file_name = "Presentation_28.xdf"
# file_name = "Presentation_29.xdf"
# file_name = "PsychoPy_21.xdf"
# file_name = "TestIO_30.xdf"
# file_name = "PTB_16.xdf"

# list streams contained in XDF file
streams = resolve_streams(file_name)

# identify the (only) EEG stream
stream_id = match_streaminfos(streams, [{"type": "EEG"}])[0]

raw = read_raw(file_name, stream_ids=[stream_id])
trigger_channel = raw.ch_names[-1]
raw.set_channel_types({trigger_channel: "stim"})
if "PS" in raw.ch_names:
    raw.set_channel_types({"PS": "stim"})
sfreq = raw.info["sfreq"]

# events from LSL markers
if trigger_channel == "DeviceTrigger":
    types = np.array([int(d) for d in raw.annotations.description])
    lsl_events = raw.annotations.onset[types % 2 == 1]  # odd numbers
else:
    lsl_events = raw.annotations.onset[raw.annotations.description == "1"]

# events from hardware triggers
hardware_events = np.where(raw.get_data(picks=trigger_channel).squeeze() > 0)[0]
# Use this line if DeviceTrigger was used with LSL Connector App < 1.20.4
# hardware_events = np.where(raw.get_data(picks=trigger_channel).squeeze() == 0)[0]


# events from photo sensor (only if there is a channel named "PS")
if "PS" in raw.ch_names:
    data = np.zeros_like(raw.times)
    d = np.append(0, np.diff(raw.get_data(picks="PS").squeeze()))
    threshold = np.quantile(d, 0.999) / 10
    tmp = np.where(d > threshold)[0]
    diff = np.full_like(tmp, 1000)
    diff[1:] = np.diff(tmp)
    data[tmp[diff > 10]] = 1
    info = mne.create_info(["Status"], sfreq, "stim")
    stim = mne.io.RawArray(data[np.newaxis, :], info)
    if len(mne.find_events(stim)) > len(hardware_events):
        photo_events = mne.find_events(stim)[:-1]  # skip the last event
    else:
        photo_events = mne.find_events(stim)

# create figure
fig, ax = plt.subplots()

# build delay figure and table
table = Table("", "min", "q25", "mean", "median", "q75", "max",
              title="Difference (ms)")

diff = (hardware_events / sfreq - lsl_events) * 1000
label = "HW – LSL"
ax.plot(diff, label=f"{label} ({diff.mean():.2f}ms ± {diff.std():.2f}ms)")
table.add_row(label, *[f"{stat:.2f}" for stat in summary(diff)])

if "PS" in raw.ch_names:
    diff = (photo_events[:, 0] / sfreq - hardware_events / sfreq) * 1000
    label = "PS – HW"
    ax.plot(diff, label=f"{label} ({diff.mean():.2f}ms ± {diff.std():.2f}ms)")
    table.add_row(label, *[f"{stat:.2f}" for stat in summary(diff)])

    diff = (photo_events[:, 0] / sfreq - lsl_events) * 1000
    label = "PS – LSL"
    ax.plot(diff, label=f"{label} ({diff.mean():.2f}ms ± {diff.std():.2f}ms)")
    table.add_row(label, *[f"{stat:.2f}" for stat in summary(diff)])

ax.set(
    title="Latencies between different modalities",
    xlabel="Events (1)",
    ylabel="Difference (ms)"
)
ax.legend()

# show table and figure
print(table)
plt.show()
