clear all
close all

%% Loading and parsing the data:
% FileName = 'EPrime_11';
% FileName = 'Presentation_26';
% FileName = 'Presentation_27';
% FileName = 'Presentation_28';
% FileName = 'Presentation_29';
% FileName = 'PsychoPy_21';
% FileName = 'TestIO_30';
FileName = 'PTB_16';

PS_Channel = 0;
% Requirements:
% -- Minimum hardware trigger channel (for LiveAmp, the last channels is taken, which either
% is the trigger channel from the STE, or the device trigger channel, if
% there is no STE). For actiCHamp it's a marker channel.
% Remember to set LSL Trigger Output Style to "EEG Channel".
% -- One LSL Marker Stream with the same markers as the hardware triggers.
% They can be integers or strings, but only values greater than 0 are taken
% into account.
% Additional:
% If a photo sensor is connected and recorded as "PS", it will be detected
% and analyzed as well, adding triggers based on the photo signal to the
% comparison.

% Load file:
% Try setting "HandleJitterRemoval" to "false" to get a better impression of bad
% signals.
LS = load_xdf([FileName,'.xdf'],'HandleJitterRemoval',true,'HandleClockSynchronization',true);

% Parsing the LSL streams and looking for a photo signal channel
for si = 1:size(LS,2)
    disp(['Stream ',num2str(si),': ',LS{si}.info.name,' from ',LS{si}.info.hostname])
    if strcmp(LS{si}.info.type,'EEG')
        LSLEEGStream = si;
        Fs = str2double(LS{si}.info.nominal_srate);
        disp(['Effective Sampling Rate: ',num2str(LS{si}.info.effective_srate)])
        for li = 1:length(LS{si}.info.desc.channels.channel)
            if find(strcmp(LS{si}.info.desc.channels.channel{li}.label, 'PS'))
                PS_Channel = li;
            end
        end
    end
    if strcmp(LS{si}.info.type,'Markers')
        LSLTriggerStream = si;
    end
end

%% Getting all the time stamps:
% The LSL marker time stamps are already directly saved in the XDF file. We
% just remove 0s (and even numbers if only the Device Trigger was used)
% and check if we need to convert to integer.
LSLTriggers_t = LS{LSLTriggerStream}.time_stamps;

if iscell(LS{LSLTriggerStream}.time_series)
    LS{LSLTriggerStream}.time_series = str2double(LS{LSLTriggerStream}.time_series);
end

if strcmp(LS{LSLEEGStream}.info.desc.channels.channel{end}.label,'DeviceTrigger')
    LSLTriggers_t(mod(LS{LSLTriggerStream}.time_series,2) == 0) = [];
else
    LSLTriggers_t(LS{LSLTriggerStream}.time_series == 0) = [];
end

% We take the last channel from the EEG stream and invert signals from the
% device trigger (if no STE was connected).
% The hardware trigger channel is -1 per default but changes to 1 or 0 for
% a single sample when a trigger was detected.
HardwareTriggers = LS{LSLEEGStream}.time_series(end,:);
% Uncomment the following block if recorded with LiveAmp LSL connector app
% version <1.20.4
% if strcmp(LS{LSLEEGStream}.info.desc.channels.channel{end}.label,'DeviceTrigger')
%     HardwareTriggers(HardwareTriggers == 1) = 42;
%     HardwareTriggers(HardwareTriggers == 0) = 1;
%     HardwareTriggers(HardwareTriggers == 42) = 0;
% end

% We are only interested in the time stamps when the signal was greater
% than 0
HardwareTriggers_t = LS{LSLEEGStream}.time_stamps(HardwareTriggers > 0);

% Using a threshold to find the time stamps for the rising edges:
% This may require some adaptation. In this case, we use a double
% differentiation, but you can also use a threshold based on the
% original signal (e.g, at 1/10 of the max. amplitude).
% This is only executed if there is an actual photo signal channel.
if PS_Channel > 0
    % Here, we take the photo signal and look for rising edges. 
    % The threshold is adapted to the signal itself
    PhotoSensor = LS{LSLEEGStream}.time_series(PS_Channel,:);
    data = zeros(1,length(PhotoSensor));
    d = [0 diff(PhotoSensor)];
    threshold = prctile(d,99.9)/10;
    tmp = find(d>threshold);
    d_tmp = 1000*ones(1, length(tmp));
    d_tmp(2:end) = diff(tmp);
    data(tmp(d_tmp > 10)) = 1;    
    data_idx = find(data == 1);
    PhotoTriggers_t = LS{LSLEEGStream}.time_stamps(data_idx);
    
    % It may be the case that the display switches back to bright at the
    % end. A trigger based on this event needs to be removed.
    if length(PhotoTriggers_t) > length(HardwareTriggers_t)
        PhotoTriggers_t(end) = [];
    end
end

%% Calculate and show all the latencies
% We compare hardware triggers versus LSL markers and also the photo
% signal, if available.
HW_minus_LSL_mean = mean(HardwareTriggers_t - LSLTriggers_t)*1000;
HW_minus_LSL_std = std(HardwareTriggers_t - LSLTriggers_t)*1000;
disp(['Hardware Triggers - LSL Software Triggers: ',num2str(HW_minus_LSL_mean),'+/-',num2str(HW_minus_LSL_std),'ms']);
if PS_Channel > 0
    PS_minus_HW_mean = mean(PhotoTriggers_t - HardwareTriggers_t)*1000;
    PS_minus_HW_std = std(PhotoTriggers_t - HardwareTriggers_t)*1000;
    PS_minus_LSL_mean = mean(PhotoTriggers_t - LSLTriggers_t)*1000;
    PS_minus_LSL_std = std(PhotoTriggers_t - LSLTriggers_t)*1000;
    disp(['PhotoSensor - Hardware Triggers: ',num2str(PS_minus_HW_mean),'+/-',num2str(PS_minus_HW_std),'ms']);
    disp(['PhotoSensor - LSL Software Triggers: ',num2str(PS_minus_LSL_mean),'+/-',num2str(PS_minus_LSL_std),'ms']);
end

% Display the differences over time (per event)
figure; xlabel('Events (1)'); ylabel('Difference (ms)');
title('Latencies between different modalities'); %ylim([-10 40])
hold on
plot((HardwareTriggers_t - LSLTriggers_t)*1000,'b');
if PS_Channel > 0
    plot((PhotoTriggers_t - HardwareTriggers_t)*1000,'r');
    plot((PhotoTriggers_t - LSLTriggers_t)*1000,'g')
    legend(['HW - LSL: ',num2str(round(HW_minus_LSL_mean,2)),'+/-',num2str(round(HW_minus_LSL_std,2)),'ms'], ...
        ['PS - HW: ',num2str(round(PS_minus_HW_mean,2)),'+/-',num2str(round(PS_minus_HW_std,2)),'ms'], ...
        ['PS - LSL: ',num2str(round(PS_minus_LSL_mean,2)),'+/-',num2str(round(PS_minus_LSL_std,2)),'ms'])
else
    legend(['HW - LSL: ',num2str(round(HW_minus_LSL_mean,2)),'+/-',num2str(round(HW_minus_LSL_std,2)),'ms'])
end



